
//  Copyright <2021> <Hans de Nivelle>
//
//  This software is released with the 3-clause BSD license, which 
//  is given below: 
// 
//  Redistribution and use in source and binary forms, with or without 
//  modification, are permitted provided that the following conditions are met:

//  Redistributions of source code must retain the above copyright notice, 
//  this list of conditions and the following disclaimer:

//  Redistributions in binary form must reproduce the above copyright notice, 
//  this list of conditions and the following disclaimer in the documentation 
//  and/or other materials provided with the distribution.
//  Neither the name of the copyright holder nor the names of its contributors 
//  may be used to endorse or promote products derived from this software 
//  without specific prior written permission.

//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
//  HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
//  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
//  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



#include "opdef.h"


const char* prolog::getcstring( optag tag )
{
   switch( tag )
   {
   case fx:
      return "fx";
   case fy:
      return "fy";
   
   case xfx:
      return "xfx";
   case xfy:
      return "xfy";
   case yfx:
      return "yfx";

   case xf:
      return "xf";
   case yf:
      return "yf";

   }
   return "???";  
}


int prolog::opdef::decide( const opdef& other ) const
{
   // Smaller number means more attraction:

   if( attraction < other.attraction )
      return -1;

   if( attraction > other.attraction )
      return 1;

   if( type == fx || type == xfx || type == yfx ) 
   {
      if( other. type == yf || other. type == yfx )
         return -1;
      return 0;
   }

   if( type == fy || type == xfy )
   {
      if( other. type == xf || other. type == xfx || other. type == xfy )
          return 1;
      return 0;
   }

   return 0;
}


bool prolog::opdef::sameside( const opdef& other ) const
{
   if( type == fx || type == fy )
      return other. type == fx || other. type == fy;

   if( type == xfx || type == xfy || type == yfx )
      return other. type == xfx || other. type == xfy || other. type == yfx;

   if( type == xf || type == yf )
      return other. type == xf || other. type == yf; 

   return false;   // unreachable, but otherwise compiler cries.  
}


bool prolog::operator == ( const opdef& def1, const opdef& def2 )
{
   return def1. str == def2. str && 
          def1. type == def2. type &&  
          def1. attraction == def2. attraction; 
}


bool prolog::operator != ( const opdef& def1, const opdef& def2 ) 
{
   return def1. str != def2. str ||
          def1. type != def2. type ||
          def1. attraction != def2. attraction; 
}


std::ostream& prolog::operator << ( std::ostream& out, const opdef& def )
{
   out << "opdef( "; 
   out << def. str << ", " << def. type << ", " << def. attraction;
   out << " )";
   return out;
}



